/*global _mediaImageFilterStrength:true */
var _mediaImageFilterStrength = 100; // keeps track of the current strength value

// For detecting IE10
document.documentElement.setAttribute('data-useragent', navigator.userAgent);

jQuery(document).ready(function($) {
    "use strict";

    // From: http://stackoverflow.com/questions/3710204/how-to-check-if-a-string-is-a-valid-json-string-in-javascript-without-using-try
    function tryParseJSON (jsonString){
        try {
            var o = JSON.parse(jsonString);

            // Handle non-exception-throwing cases:
            // Neither JSON.parse(false) or JSON.parse(1234) throw errors, hence the type-checking,
            // but... JSON.parse(null) returns 'null', and typeof null === "object",
            // so we must check for that, too.
            if (o && typeof o === "object" && o !== null) {
                return o;
            }
        }
        catch (e) { }

        return false;
    }

    function basename(path) {
       return path.split(/[\\/]/).pop();
    }


    /*
     * Apply filters
     */

    var _mediaImageFilterSaveTimeout = null;
    var _mediaImageFilterOrigImageData = null;
    function mediaImageFilterApplyFilter( selector, filter ) {
        Caman.Event.listen(
            Caman(selector, function () {
                _mediaImageFilterOrigImageData = $('#media-image-filter-large-preview canvas')[0].getContext('2d').getImageData(
                    0,
                    0,
                    $('#media-image-filter-large-preview canvas')[0].width,
                    $('#media-image-filter-large-preview canvas')[0].height
                ).data;
                this[filter]();
                this.render();
            }),
            "blockFinished",
            function () {
                if ( _mediaImageFilterSaveTimeout !== null ) {
                    clearTimeout( _mediaImageFilterSaveTimeout );
                    _mediaImageFilterSaveTimeout = null;
                }
                _mediaImageFilterSaveTimeout = setTimeout(function() {
                    mediaImageApplyStrength();
                    _mediaImageFilterSaveTimeout = null;
                }, 700);
            }
        );
    }

    $('body').on('customRenderFinsihed', '#media-image-filter-large-preview canvas', function() {
        _mediaImageFilterOrigImageData = null;
        $('#media-image-filter-large-preview canvas').trigger('filterRenderFinished');
        $('.media-image-filter-spinner').fadeOut();
        $('.media-image-filter-save').fadeIn();
    });

    // Only applies to the camanjs presets
    // var _mediaImageFilterPrevStrength = 100;
    function mediaImageApplyStrength() {
        var canvas = $('#media-image-filter-large-preview canvas')[0];
        var context = canvas.getContext('2d');

        var filteredImagedata = context.getImageData(0, 0, canvas.width, canvas.height);

        var pixels = 4 * canvas.width * canvas.height;
        var data1 = filteredImagedata.data;

        var imageData = context.createImageData( canvas.width, canvas.height );
        var ratio = 0.8;
        ratio = $(".media-image-filter-strength-slider").val() / 100;
        while ( pixels-- ) {
            imageData.data[ pixels ] = data1[ pixels ] * ratio + _mediaImageFilterOrigImageData[ pixels ] * ( 1 - ratio );
        }

        context.putImageData( imageData, 0, 0 );

        _mediaImageFilterOrigImageData = null;

        $('#media-image-filter-large-preview canvas').trigger('filterRenderFinished');
        $('.media-image-filter-spinner').fadeOut();
        $('.media-image-filter-save').fadeIn();
    }

    /*
     * Opens the preview window and shows the save button
     */

    function openPreviewWindow() {
        if ( $('#media-image-filter-large-preview').length > 0 ) {
            $('#media-image-filter-large-preview').remove();
        }
        if ( typeof wp.media.frame.content.get('media').options.selection.models[0] === 'undefined' ) {
            return;
        }

        var largeImage = wp.media.frame.content.get('media').sidebar._views.details.model.attributes.url;

        $('<div></div>').attr('id', 'media-image-filter-large-preview').prependTo('.attachments-browser')
        .append('<div class="media-image-filter-controls-wrapper"><div class="media-image-filter-controls"><span class="media-image-filter-strength-label">Strength:</span><div class="media-image-filter-strength-slider"></div><img class="media-image-filter-spinner" src="' + GambitMediaImageFilter.spinner_gif + '"/><a href="#" class="media-image-filter-save button media-button button-secondary button-large" style="display: none">' + GambitMediaImageFilter.save_label + '</a><a href="#" class="media-image-filter-large-preview-close"><span class="media-modal-icon"></span></a></div></div>')
        .append('<img src="' + largeImage + '"/>');

        // Create the strength slider
        $(".media-image-filter-strength-slider").noUiSlider({
           	start: parseInt( _mediaImageFilterStrength ),
            connect: "lower",
            step: 1,
           	range: {
           		'min': 0,
           		'max': 100
           	}
        });
    }


    $('body').on('change', '.media-image-filter-strength-slider', function() {
        _mediaImageFilterStrength = $(this).val();
        var curFiter = $('#media-image-filter-large-preview').attr('data-filter');
        $('a[data-filter=' + curFiter + ']').attr('data-keep-strength', '1').trigger('click');
    });


    /*
     * When the save filtered image button is clicked, get the contents of the canvas
     * and ajax-save it as a new attachment
     */

    $('body').on('click', '.media-image-filter-save', function() {
        $('.media-image-filter-save').attr('disabled', 'disabled').text( GambitMediaImageFilter.wait_label );
        $('.media-image-filter-spinner').fadeIn();

        var largeImage = wp.media.frame.content.get('media').sidebar._views.details.model.attributes.url;
        var format = /(jpg|JPG|JPEG|jpeg)$/.test(largeImage) ? 'image/jpeg' : 'image/png';
        format = /(gif|GIF)$/.test(format) ? 'image/gif' : format;

        var imageData = $('#media-image-filter-large-preview canvas')[0].toDataURL(format);

        $.post( GambitMediaImageFilter.ajax_url,
            {
                action: 'save_filtered_image',
                image: imageData,
                format: format,
                title: wp.media.frame.content.get('media').sidebar._views.details.model.attributes.title,
                name: wp.media.frame.content.get('media').sidebar._views.details.model.attributes.name,
                filename: largeImage,
                filter: $('#media-image-filter-large-preview').attr('data-filter'),
                nonce: GambitMediaImageFilter.nonce
            },
            function ( data ){

                // We either get the attachment details as json, or an error message
                var jsonData = tryParseJSON( data );

                if ( jsonData !== false ) {

                    /*
                     * Create an Attachment object containing the filtered image's data, and manually insert it into
                     * the library
                     */

                    var library = wp.media.frame.state().get('library');

                    var newAttachment = new wp.media.model.Attachment();

                    var sizes = {
                        full: {
                            height: jsonData.sizes_data.height,
                            orientation: "landscape",
                            url: jsonData.attachment_url,
                            width: jsonData.sizes_data.width
                        }
                    };

                    if ( typeof jsonData.sizes_data.sizes.large !== 'undefined' ) {
                        sizes.large = {
                            height: jsonData.sizes_data.sizes.large.height,
                            orientation: "landscape",
                            url: jsonData.attachment_url.replace( basename( jsonData.attachment_url ), '' ) + jsonData.sizes_data.sizes.large.file,
                            width: jsonData.sizes_data.sizes.large.width
                        };
                    }

                    if ( typeof jsonData.sizes_data.sizes.medium !== 'undefined' ) {
                        sizes.medium = {
                            height: jsonData.sizes_data.sizes.medium.height,
                            orientation: "landscape",
                            url: jsonData.attachment_url.replace( basename( jsonData.attachment_url ), '' ) + jsonData.sizes_data.sizes.medium.file,
                            width: jsonData.sizes_data.sizes.medium.width
                        };
                    }

                    if ( typeof jsonData.sizes_data.sizes.thumbnail !== 'undefined' ) {
                        sizes.thumbnail = {
                            height: jsonData.sizes_data.sizes.thumbnail.height,
                            orientation: "landscape",
                            url: jsonData.attachment_url.replace( basename( jsonData.attachment_url ), '' ) + jsonData.sizes_data.sizes.thumbnail.file,
                            width: jsonData.sizes_data.sizes.thumbnail.width
                        };
                    }

                    var nonces = {
                        delete: jsonData.delete_nonce,
                        edit: jsonData.edit_nonce,
                        update: jsonData.update_nonce
                    };

                    newAttachment.id = jsonData.id;
                    newAttachment.attributes.alt = "";
                    newAttachment.attributes.author = "0";
                    newAttachment.attributes.caption = "";
                    newAttachment.attributes.compat = jsonData.compat_fields;
                    newAttachment.attributes.date = new Date();
                    newAttachment.attributes.dateFormatted = ["January", "February", "March", "April", "May", "June",
                    "July", "August", "September", "October", "November", "December"][(new Date()).getMonth()] + ' ' + (new Date()).getDate() + ', ' + (new Date()).getFullYear();
                    newAttachment.attributes.description = "";
                    newAttachment.attributes.editLink = GambitMediaImageFilter.admin_post_url + "?post=" + jsonData.id + "&action=edit";
                    newAttachment.attributes.filename = basename( jsonData.attachment_url );
                    newAttachment.attributes.height = jsonData.sizes_data.height;
                    newAttachment.attributes.icon = GambitMediaImageFilter.media_default_url;
                    newAttachment.attributes.id = jsonData.id;
                    newAttachment.attributes.link = jsonData.attachment_link;
                    newAttachment.attributes.menuOrder = 0;
                    newAttachment.attributes.mime = jsonData.attachment_data.post_mime_type;
                    newAttachment.attributes.modified = new Date();
                    newAttachment.attributes.name = jsonData.attachment_data.post_name;
                    newAttachment.attributes.nonces = nonces;
                    newAttachment.attributes.orientation = "landscape";
                    newAttachment.attributes.sizes = sizes;
                    newAttachment.attributes.status = "inherit";
                    newAttachment.attributes.subtype = basename( jsonData.attachment_data.post_mime_type );
                    newAttachment.attributes.title = jsonData.attachment_data.post_title;
                    newAttachment.attributes.type = "image";
                    newAttachment.attributes.uploadedTo = 0;
                    newAttachment.attributes.url = jsonData.attachment_url;
                    newAttachment.attributes.width = jsonData.sizes_data.width;

                    // Add in our current list
                    library.models.unshift( newAttachment );

                    // Update the list
                    library.reset( library.models );

                    // Select the new attachment
                    wp.media.frame.state().get('selection').reset();
                    wp.media.frame.state().get('selection').add( newAttachment );

                } else {
                    // something went wrong, ask to manually save and upload
                    if ( data === '' ) {
                        data = GambitMediaImageFilter.unknown_error;
                    }

                    alert( data + "\n\n" + GambitMediaImageFilter.manual_label );
                    window.open(imageData);
                }

                $('.media-image-filter-spinner').fadeOut();
                $('.media-image-filter-large-preview-close').trigger('click');
                $('.media-image-filter-save').removeAttr('disabled').text( GambitMediaImageFilter.save_label );
            }
        );
    });


   /*
    * When the large preview area is closed, hide the save button and reset the select filter
    */

    $('body').on('click', '.media-image-filter-large-preview-close', function() {
        $('#media-image-filter-large-preview').remove();
        $('.media-image-filter-save').hide();
        $('#media-image-filter-large-preview').attr('data-filter', '');
        return false;
    });


    $('body').on('filterRenderFinished', 'canvas', function() {
    });





    /*
     * Apply the selected filter
     */

    var _mediaImageFilterApplyOnce = false;
    $('body').on('click', 'a[data-filter]', function() {

        // When called from the media library, this is somehow called twice. Only entertain the first call
        if ( _mediaImageFilterApplyOnce ) {
            return;
        }
        _mediaImageFilterApplyOnce = true;

        $('.media-image-filter-spinner').fadeIn();
        $('.media-image-filter-save').fadeOut();

        var filter;
        if ( $(this).is('a') ) {
            filter = $(this).attr('data-filter');
        } else {
            filter = $(this).val();
        }

        if ( typeof $(this).attr('data-keep-strength') === 'undefined' ) {
            _mediaImageFilterStrength = 100;
        }

        $(this).removeAttr('data-keep-strength');

        openPreviewWindow();

        $('#media-image-filter-large-preview').attr('data-filter', filter);

        // For IE 10 & IE 9, rendering needs to be done twice for the image to show (IE10 BUG)
        if ( $("html[data-useragent*='MSIE 10.0'], html[data-useragent*='MSIE 9.0']").length > 0 ) {
            Caman('#media-image-filter-large-preview > img, #media-image-filter-large-preview canvas', function () {

                this.render();
            });

            openPreviewWindow();

            if ( filter !== 'original' ) {
                setTimeout(function() {
                    mediaImageFilterApplyFilter(
                        '#media-image-filter-large-preview > img, #media-image-filter-large-preview canvas',
                        filter
                    );
                    _mediaImageFilterApplyOnce = false;
                }, 1000);
            }
        } else {
            if ( filter !== 'original' ) {
                setTimeout(function() {
                    mediaImageFilterApplyFilter(
                        '#media-image-filter-large-preview > img, #media-image-filter-large-preview canvas',
                        filter
                    );
                    _mediaImageFilterApplyOnce = false;
                }, 500);
            }
        }

        if ( filter === 'original' ) {
            $('.media-image-filter-spinner').fadeOut();
            $('.media-image-filter-save').fadeOut();
        }

        return false;
    });
});