<?php
/*
Plugin Name: Image Maker Bot
Description: Image filters and loading animation effects; replacement for predecessor, WP Aviari
Author: Vas B., J.B.
Version: 2.0
Author URI: http://mastergigs.com
*/

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

defined( 'GAMBIT_MEDIA_IMAGE_FILTERS' ) or define( 'GAMBIT_MEDIA_IMAGE_FILTERS', 'gambit-media-image-filter' );

class GambitMediaImageFilter {

    function __construct() {
        add_action( 'admin_enqueue_scripts', array( $this, 'addFilterFieldStyles' ) );
        add_filter( 'attachment_fields_to_edit', array( $this, 'addFilterFields' ), 10, 2 );

        add_action( 'wp_ajax_save_filtered_image', array( $this, 'saveFilteredImage' ) );
		add_action( 'plugins_loaded', array( $this, 'loadTextDomain' ), 1 );
        add_action( 'admin_head', array( $this, 'addImageFiltersInMediaLibrary' ) );
    }


	/**
	 * Loads the translations
	 *
	 * @return	void
	 * @since	1.0
	 */
	public function loadTextDomain() {
		load_plugin_textdomain( GAMBIT_MEDIA_IMAGE_FILTERS, false, basename( dirname( __FILE__ ) ) . '/languages/' );
	}

    public function addImageFiltersInMediaLibrary() {
        $screen = get_current_screen();

        if ( empty( $screen->post_type ) ) {
            return;
        }
        if ( $screen->post_type != 'attachment' ) {
            return;
        }

        wp_enqueue_media();
        wp_enqueue_script( 'custom-header' );

        $pluginData = get_plugin_data( __FILE__ );
        wp_enqueue_script( 'media-image-filter-attachment', plugins_url( 'js/script-attachment-ck.js', __FILE__ ), array( 'jquery' ), $pluginData['Version'] );
        wp_localize_script( 'media-image-filter-attachment', __CLASS__ . 'Library', array(
            'attachment_id' => $_GET['post'],
            'apply_label' => __( 'Apply image filters', GAMBIT_MEDIA_IMAGE_FILTERS ),
        ) );
    }

    public function addFilterFieldStyles() {
        $pluginData = get_plugin_data( __FILE__ );
        wp_enqueue_style( 'caman', plugins_url( 'css/style.css', __FILE__ ), array(), $pluginData['Version'] );

        wp_enqueue_script( 'caman', plugins_url( 'js/caman.full-ck.js', __FILE__ ), array( 'jquery' ), $pluginData['Version'] );
        wp_enqueue_script( 'caman-instagram', plugins_url( 'js/caman.instagram-ck.js', __FILE__ ), array( 'caman' ), $pluginData['Version'] );
        wp_enqueue_script( 'caman-gambit', plugins_url( 'js/caman.gambit-ck.js', __FILE__ ), array( 'caman-instagram' ), $pluginData['Version'] );

        wp_enqueue_style( 'nouislider', plugins_url( 'css/jquery.nouislider.css', __FILE__ ), array(), $pluginData['Version'] );
        wp_enqueue_script( 'nouislider', plugins_url( 'js/jquery.nouislider.min.js', __FILE__ ), array( 'jquery' ), $pluginData['Version'] );

        wp_enqueue_script( 'media-image-filter', plugins_url( 'js/script-ck.js', __FILE__ ), array( 'jquery' ), $pluginData['Version'] );
        wp_localize_script( 'media-image-filter', __CLASS__, array(
            'ajax_url' => admin_url( 'admin-ajax.php' ),
            'nonce' => wp_create_nonce( __CLASS__ ),
            'wait_label' => __( 'Saving, please wait...', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'unknown_error' => __( 'Error: An unknown error occurred, please try again.', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'manual_label' => __( 'The filtered image will instead open in a new window. Right click and save the image to you computer first, then upload it to the media manager in WordPress', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'save_label' => __( 'Save Filtered Image', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'spinner_gif' => admin_url( '/images/spinner.gif' ),
            'admin_post_url' => admin_url( 'post.php' ),
            'media_default_url' => includes_url( 'images/media/default.png' ),
        ) );
    }

    public function saveFilteredImage() {
        check_ajax_referer( __CLASS__, 'nonce' );

        $ext = $_POST['format'];
        if ( $ext == 'image/png' ) {
            $ext = 'png';
        } else if ( $ext == 'image/gif' ) {
            $ext = 'gif';
        } else {
            $ext = 'jpg';
        }

        $imageData = $_POST['image'];
        // remove prefix
        $imageData = substr( $imageData, strpos( $imageData, "," ) + 1 );
        $imageData = base64_decode( $imageData );

        $fileInfo = pathinfo( $_POST['filename'] );
        $fileName = basename( $_POST['filename'], '.' . $fileInfo['extension'] );

		$uploads = wp_upload_dir();
		$uploadsFolder = trailingslashit( $uploads['path'] );
		$newFileName = $uploadsFolder . $fileName . '-' . $_POST['filter'] . '.' . $ext;

		WP_Filesystem();
		global $wp_filesystem;

        // Find a good filename that doesn't exist
        $i = 1;
        while ( $wp_filesystem->exists( $newFileName ) ) {
            $newFileName = $uploadsFolder . $fileName . '-' . $_POST['filter'] . $i++ . '.' . $ext;
        }

		// Verify that we can create the file
		if ( $wp_filesystem->exists( $newFileName ) ) {
			if ( ! $wp_filesystem->is_writable( $newFileName ) ) {
                printf( __( 'Error: the file %s is not writable', GAMBIT_MEDIA_IMAGE_FILTERS ), $newFileName );
				die();
			}
			if ( ! $wp_filesystem->is_readable( $newFileName ) ) {
                printf( __( 'Error: the file %s cannot be read', GAMBIT_MEDIA_IMAGE_FILTERS ), $newFileName );
				die();
			}
		}
		// Verify directory
		if ( ! $wp_filesystem->is_dir( dirname( $newFileName ) ) ) {
            printf( __( 'Error: the folder %s cannot be found', GAMBIT_MEDIA_IMAGE_FILTERS ), dirname( $newFileName ) );
			die();
		}
		if ( ! $wp_filesystem->is_writable( dirname( $newFileName ) ) ) {
            printf( __( 'Error: the folder %s is not writable', GAMBIT_MEDIA_IMAGE_FILTERS ), dirname( $newFileName ) );
			die();
		}

		if ( ! $wp_filesystem->put_contents( $newFileName, $imageData, 0644 ) ) {
            printf( __( 'Error: the image %s could not be saved', GAMBIT_MEDIA_IMAGE_FILTERS ), $newFileName );
		    die();
		}


        /*
         * Save attachment
         */

        $siteurl = get_option( 'siteurl' );
		$file_info = getimagesize( $newFileName );

		//create an array of attachment data to insert into wp_posts table
		$tableParams = array(
			'post_author' => get_current_user_id(),
			'post_date' => current_time( 'mysql' ),
			'post_date_gmt' => current_time( 'mysql' ),
			'post_title' => $_POST['title'] . ' ' . $_POST['filter'],
			'post_status' => 'inherit',
			'comment_status' => 'closed',
			'ping_status' => 'closed',
			'post_name' => sanitize_title_with_dashes( str_replace( "_", "-", $_POST['name'] . '-' . $_POST['filter'] ) ),
            'post_modified' => current_time( 'mysql' ),
			'post_modified_gmt' => current_time( 'mysql' ),
			'post_parent' => 0,
			'post_type' => 'attachment',
			'guid' => $newFileName,
			'post_mime_type' => $file_info['mime'],
			'post_excerpt' => '',
			'post_content' => ''
		);

		// insert the database record
		$attachmentID = wp_insert_attachment( $tableParams, $newFileName, 0 );

		// generate metadata and thumbnails
		if ( $attachmentData = wp_generate_attachment_metadata( $attachmentID, $newFileName ) ) {
			wp_update_attachment_metadata( $attachmentID, $attachmentData );
		}

        // We need these data to create a new attachment object in javascript to update the media library
        echo json_encode( array(
            'id' => $attachmentID,
            'attachment_data' => $tableParams,
            'sizes_data' => $attachmentData,
            'attachment_url' => wp_get_attachment_url( $attachmentID ),
            'attachment_link' => get_attachment_link( $attachmentID ),
            'delete_nonce' => wp_create_nonce( 'delete-post_' . $attachmentID ),
            'update_nonce' => wp_create_nonce( 'update-post_' . $attachmentID ),
            'edit_nonce' => wp_create_nonce( 'image_editor-' . $attachmentID ),
            'compat_fields' => get_compat_media_markup( $attachmentID ),
        ) );

        die();
    }

    /**
     * No actual fields are saved here.
     */
    public function addFilterFields( $form_fields, $post ) {
        $filters = array(
            'vintage' => __( 'Vintage', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'lomo' => __( 'Lomo', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'clarity' => __( 'Clarity', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'sinCity' => __( 'Sin City', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'sunrise' => __( 'Sunrise', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'crossProcess' => __( 'Cross Process', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'greyscale' => __( 'Greyscale', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'orangePeel' => __( 'Orange Peel', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'love' => __( 'Love', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'grungy' => __( 'Grungy', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'jarques' => __( 'Jarques', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'pinhole' => __( 'Pinhole', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'oldBoot' => __( 'OldBoot', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'glowingSun' => __( 'Glowing Sun', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'hazyDays' => __( 'Hazy Days', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'herMajesty' => __( 'Her Majesty', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'nostalgia' => __( 'Nostalgia', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'hemingway' => __( 'Hemingway', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'concentrate' => __( 'Concentrate', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'nashville' => __( 'Nashville', GAMBIT_MEDIA_IMAGE_FILTERS ),
            '_1977' => __( '1977', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'brannan' => __( 'Brannan', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'gotham' => __( 'Gotham', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'hefe' => __( 'Hefe', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'lordkelvin' => __( 'Lord Kelvin', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'xpro' => __( 'X-PRO II', GAMBIT_MEDIA_IMAGE_FILTERS ),
            'tenderpetals' => __( 'Tender Petals', GAMBIT_MEDIA_IMAGE_FILTERS ), // by onixia
            'coffee' => __( 'Coffee', GAMBIT_MEDIA_IMAGE_FILTERS ), // by onixia
            'trains' => __( 'Trains', GAMBIT_MEDIA_IMAGE_FILTERS ), // by onixia
            'oldschool' => __( 'Old School', GAMBIT_MEDIA_IMAGE_FILTERS ), // by onixia
            'freshblue' => __( 'Fresh Blue', GAMBIT_MEDIA_IMAGE_FILTERS ), // by onixia
            'fogyblue' => __( 'Fogy Blue', GAMBIT_MEDIA_IMAGE_FILTERS ), // by onixia
            'country' => __( 'Country', GAMBIT_MEDIA_IMAGE_FILTERS ), // by onixia
            'fapurple' => __( 'Purple', GAMBIT_MEDIA_IMAGE_FILTERS ), // by Frikialternatiivo
            'fablue' => __( 'Blue', GAMBIT_MEDIA_IMAGE_FILTERS ), // by Frikialternatiivo
            'faretro' => __( 'Retro', GAMBIT_MEDIA_IMAGE_FILTERS ), // by Frikialternatiivo
            'intensify' => __( 'Intensify', GAMBIT_MEDIA_IMAGE_FILTERS ), // by all17
            'everglow' => __( 'Everglow', GAMBIT_MEDIA_IMAGE_FILTERS ), // by alais-stock
            'frozen' => __( 'Frozen', GAMBIT_MEDIA_IMAGE_FILTERS ), // by biszkopciik
            'fall' => __( 'Fall', GAMBIT_MEDIA_IMAGE_FILTERS ), // by biszkopciik
            'summer' => __( 'Summer', GAMBIT_MEDIA_IMAGE_FILTERS ), // by biszkopciik
            'sunset' => __( 'Sunset', GAMBIT_MEDIA_IMAGE_FILTERS ), // by biszkopciik
            'moonstone' => __( 'Moonstone', GAMBIT_MEDIA_IMAGE_FILTERS ), // by FrenchLilac
            'washed' => __( 'Washed', GAMBIT_MEDIA_IMAGE_FILTERS ), // by FrenchLilac
            'fourshores' => __( 'Four Shores', GAMBIT_MEDIA_IMAGE_FILTERS ), // by FrenchLilac
            'letitrain' => __( 'Let It Rain', GAMBIT_MEDIA_IMAGE_FILTERS ), // by FrenchLilac
        );
        asort( $filters );
        $filters = array_merge( array( 'original' => __( 'Original', GAMBIT_MEDIA_IMAGE_FILTERS ) ), $filters );

        $select = '<option value="">' . __( '- Select an image filter -', GAMBIT_MEDIA_IMAGE_FILTERS ) . '</option>';
        $a = "";
        foreach ( $filters as $key => $name ) {
            $select .= "<option value='{$key}'>{$name}</option>\n";
            $a .= "<a href='#' data-filter='{$key}' class='media-image-filter-choice' style='background-image: url(\"" . plugins_url( "images/{$key}.jpg", __FILE__ ) . "\")'><span>{$name}</span></a>";
        }

        $form_fields['media-image-filter'] = array(
            'label' => '<h3>' . __( 'Media Image Filter', GAMBIT_MEDIA_IMAGE_FILTERS ) . '</h3>',
            'input' => 'html',
            'html' =>
                '<p class="help">'
                    . __( "Choose a filter first, afterwards click on the <strong>Save Filtered Image</strong> button that appears to save the image to the media manager.", GAMBIT_MEDIA_IMAGE_FILTERS )
                . '</p>'
                . $a
        );
        return $form_fields;
    }

}

new GambitMediaImageFilter();